
import base_section

from runtest import conditions

class ComputeFeatureEnabled(base_section.BaseSection):

    name = "compute-feature-enabled"
    options = [
        'api_extensions',
        'attach_encrypted_volume',
        'block_migrate_cinder_iscsi',
        'block_migration_for_live_migration',
        'change_password',
        'cold_migration',
        'config_drive',
        'console_output',
        'disk_config',
        'enable_instance_password',
        'interface_attach',
        'live_migrate_back_and_forth',
        'live_migration',
        'metadata_service',
        'nova_cert',
        'pause',
        'personality',
        'rdp_console',
        'rescue',
        'resize',
        'scheduler_available_filters',
        'serial_console',
        'shelve',
        'snapshot',
        'spice_console',
        'suspend',
        'swap_volume',
        'vnc_console',
        'vnc_server_header',
    ]


    @property
    def api_extensions(self):
        pass

    @property
    def attach_encrypted_volume(self):
        compute_check = conditions.BaseRule('*.nova.compute.enabled', 'eq', True)
        cinder_check = conditions.BaseRule('*.cinder.controller.enabled', 'eq', True)
        barbican_check = conditions.BaseRule('*.barbican.server.enabled', 'eq', True)
        if compute_check.check(self.pillar) and cinder_check.check(self.pillar) and barbican_check.check(self.pillar):
            c = conditions.BaseRule('nova.controller.enabled', 'eq', True)
            return self.get_item_when_condition_match('nova.controller.barbican.enabled', c)
        return False

    @property
    def block_migrate_cinder_iscsi(self):
        c = conditions.BaseRule('cinder.volume.enabled', 'eq', True)
        backends = self.get_item_when_condition_match(
            'cinder.volume.backend', c)
        if not backends:
            return False

        for backend_name,backend in backends.iteritems():
            res = backend['engine']
            if res == 'lvm':
              return True
        return False


    @property
    def block_migration_for_live_migration(self):
        # Ironic does not support block migration for live migration
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False
        c = conditions.BaseRule('cinder.volume.enabled', 'eq', True)
        backends = self.get_item_when_condition_match(
            'cinder.volume.backend', c)
        if not backends:
            return False

        for backend_name,backend in backends.iteritems():
            res = backend['engine']
            if res == 'lvm':
              return True
        return False

    @property
    def change_password(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def cold_migration(self):
        pass

    @property
    def config_drive(self):
        pass

    @property
    def console_output(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def disk_config(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def enable_instance_password(self):
        pass

    @property
    def interface_attach(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def live_migrate_back_and_forth(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def live_migration(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False
        return conditions.BaseRule('*.nova.compute.enabled', 'eq', True,
                                   multiple='multiple').check(self.pillar)

    @property
    def metadata_service(self):
        pass

    @property
    def nova_cert(self):
        pass

    @property
    def pause(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def personality(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def rdp_console(self):
        pass

    @property
    def rescue(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def resize(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False
        # NOTE(vsaienko) allow_resize_to_same_host is hardcoded to True in
        # nova formula, update when value is configurable.
        res = conditions.BaseRule('*.nova.compute.enabled', 'eq', True,
                            multiple='any').check(self.pillar)
        return res

    @property
    def scheduler_available_filters(self):
        pass

    @property
    def serial_console(self):
        pass

    @property
    def shelve(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def snapshot(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def spice_console(self):
        pass

    @property
    def suspend(self):
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False

    @property
    def swap_volume(self):
        pass

    @property
    def vnc_console(self):
        # The vnc is enabled by default in nova-formula, so rely on compute presence.
        #TODO: fix when value is unhardcoded.
        c = conditions.BaseRule('ironic.conductor.enabled', 'eq', True)
        ironic = self.get_item_when_condition_match(
            'ironic.conductor.enabled', c)
        if ironic:
            return False
        return conditions.BaseRule('*.nova.compute.enabled', 'eq', True,
                            multiple='any').check(self.pillar)

    @property
    def vnc_server_header(self):
        if conditions.BaseRule('*.nginx.server.enabled', 'eq', True,
                            multiple='any').check(self.pillar):
            return 'nginx'
